@@ -1,10 +1,12 @@
 import ast
 import asyncio
+import inspect
 from typing import Any, Dict, List, Optional
 
 from .interpreter_core import ASTInterpreter
 from .exceptions import ReturnException
 
+
 class GeneratorWrapper:
     def __init__(self, gen):
         self.gen = gen
@@ -44,6 +46,7 @@ class GeneratorWrapper:
         self.closed = True
         self.gen.close()
 
+
 class Function:
     def __init__(self, node: ast.FunctionDef, closure: List[Dict[str, Any]], interpreter: ASTInterpreter,
                  pos_kw_params: List[str], vararg_name: Optional[str], kwonly_params: List[str],
@@ -51,6 +54,8 @@ class Function:
         self.node: ast.FunctionDef = node
         self.closure: List[Dict[str, Any]] = closure[:]
         self.interpreter: ASTInterpreter = interpreter
+        # Fix: Support positional-only parameters
+        self.posonly_params = [arg.arg for arg in node.args.posonlyargs] if hasattr(node.args, 'posonlyargs') else []
         self.pos_kw_params = pos_kw_params
         self.vararg_name = vararg_name
         self.kwonly_params = kwonly_params
@@ -59,28 +64,38 @@ class Function:
         self.kw_defaults = kw_defaults
         self.defining_class = None
         self.is_generator = any(isinstance(n, (ast.Yield, ast.YieldFrom)) for n in ast.walk(node))
-        self.generator_state = None  # Added for generator protocol
 
     async def __call__(self, *args: Any, **kwargs: Any) -> Any:
         new_env_stack: List[Dict[str, Any]] = self.closure[:]
         local_frame: Dict[str, Any] = {}
         local_frame[self.node.name] = self
 
-        num_pos = len(self.pos_kw_params)
+        # Fix: Handle positional-only parameters
+        num_posonly = len(self.posonly_params)
+        num_pos_kw = len(self.pos_kw_params)
+        total_pos = num_posonly + num_pos_kw
+        pos_args_used = 0
+
         for i, arg in enumerate(args):
-            if i < num_pos:
-                local_frame[self.pos_kw_params[i]] = arg
+            if i < num_posonly:
+                local_frame[self.posonly_params[i]] = arg
+                pos_args_used += 1
+            elif i < total_pos:
+                local_frame[self.pos_kw_params[i - num_posonly]] = arg
+                pos_args_used += 1
             elif self.vararg_name:
                 if self.vararg_name not in local_frame:
                     local_frame[self.vararg_name] = []
                 local_frame[self.vararg_name].append(arg)
             else:
-                raise TypeError(f"Function '{self.node.name}' takes {num_pos} positional arguments but {len(args)} were given")
+                raise TypeError(f"Function '{self.node.name}' takes {total_pos} positional arguments but {len(args)} were given")
         if self.vararg_name and self.vararg_name not in local_frame:
             local_frame[self.vararg_name] = tuple()
 
         for kwarg_name, kwarg_value in kwargs.items():
-            if kwarg_name in self.pos_kw_params or kwarg_name in self.kwonly_params:
+            if kwarg_name in self.posonly_params:
+                raise TypeError(f"Function '{self.node.name}' got an unexpected keyword argument '{kwarg_name}' (positional-only)")
+            elif kwarg_name in self.pos_kw_params or kwarg_name in self.kwonly_params:
                 if kwarg_name in local_frame:
                     raise TypeError(f"Function '{self.node.name}' got multiple values for argument '{kwarg_name}'")
                 local_frame[kwarg_name] = kwarg_value
@@ -101,7 +116,8 @@ class Function:
         if self.kwarg_name and self.kwarg_name in local_frame:
             local_frame[self.kwarg_name] = dict(local_frame[self.kwarg_name])
 
-        missing_args = [param for param in self.pos_kw_params if param not in local_frame and param not in self.pos_defaults]
+        missing_args = [param for param in self.posonly_params if param not in local_frame]
+        missing_args += [param for param in self.pos_kw_params if param not in local_frame and param not in self.pos_defaults]
         missing_args += [param for param in self.kwonly_params if param not in local_frame and param not in self.kw_defaults]
         if missing_args:
             raise TypeError(f"Function '{self.node.name}' missing required arguments: {', '.join(missing_args)}")
@@ -118,19 +134,26 @@ class Function:
             new_interp.current_instance = args[0]
 
         if self.is_generator:
-            async def generator():
+            # Updated: Return a proper synchronous generator for sync functions
+            def generator():
                 for body_stmt in self.node.body:
                     if isinstance(body_stmt, ast.Expr) and isinstance(body_stmt.value, ast.Yield):
-                        value = await new_interp.visit(body_stmt.value, wrap_exceptions=True)
+                        # Use asyncio.run to handle any awaits in the yield value
+                        value = asyncio.run(new_interp.visit(body_stmt.value, wrap_exceptions=True))
                         yield value
                     elif isinstance(body_stmt, ast.Expr) and isinstance(body_stmt.value, ast.YieldFrom):
-                        sub_iterable = await new_interp.visit(body_stmt.value, wrap_exceptions=True)
-                        for v in sub_iterable:
-                            yield v
+                        sub_iterable = asyncio.run(new_interp.visit(body_stmt.value, wrap_exceptions=True))
+                        if hasattr(sub_iterable, '__aiter__'):
+                            raise TypeError("cannot 'yield from' an asynchronous iterable in a synchronous generator")
+                        else:
+                            for v in sub_iterable:
+                                yield v
+                    elif isinstance(body_stmt, ast.Return):
+                        value = asyncio.run(new_interp.visit(body_stmt.value, wrap_exceptions=True)) if body_stmt.value else None
+                        raise StopIteration(value)
                     else:
-                        await new_interp.visit(body_stmt, wrap_exceptions=True)
-            gen = generator()
-            return GeneratorWrapper(gen)
+                        asyncio.run(new_interp.visit(body_stmt, wrap_exceptions=True))
+            return GeneratorWrapper(generator())
         else:
             last_value = None
             try:
@@ -161,6 +184,7 @@ class Function:
         method.__self__ = instance
         return method
 
+
 class AsyncFunction:
     def __init__(self, node: ast.AsyncFunctionDef, closure: List[Dict[str, Any]], interpreter: ASTInterpreter,
                  pos_kw_params: List[str], vararg_name: Optional[str], kwonly_params: List[str],
@@ -168,6 +192,8 @@ class AsyncFunction:
         self.node: ast.AsyncFunctionDef = node
         self.closure: List[Dict[str, Any]] = closure[:]
         self.interpreter: ASTInterpreter = interpreter
+        # Fix: Support positional-only parameters
+        self.posonly_params = [arg.arg for arg in node.args.posonlyargs] if hasattr(node.args, 'posonlyargs') else []
         self.pos_kw_params = pos_kw_params
         self.vararg_name = vararg_name
         self.kwonly_params = kwonly_params
@@ -176,27 +202,35 @@ class AsyncFunction:
         self.kw_defaults = kw_defaults
 
     async def __call__(self, *args: Any, _return_locals: bool = False, **kwargs: Any) -> Any:
-        """Execute the async function. If _return_locals is True, return a tuple (result, local_vars).
-        Otherwise, return only the result. This allows capturing local variables at the top level."""
         new_env_stack: List[Dict[str, Any]] = self.closure[:]
         local_frame: Dict[str, Any] = {}
         local_frame[self.node.name] = self
 
-        num_pos = len(self.pos_kw_params)
+        num_posonly = len(self.posonly_params)
+        num_pos_kw = len(self.pos_kw_params)
+        total_pos = num_posonly + num_pos_kw
+        pos_args_used = 0
+
         for i, arg in enumerate(args):
-            if i < num_pos:
-                local_frame[self.pos_kw_params[i]] = arg
+            if i < num_posonly:
+                local_frame[self.posonly_params[i]] = arg
+                pos_args_used += 1
+            elif i < total_pos:
+                local_frame[self.pos_kw_params[i - num_posonly]] = arg
+                pos_args_used += 1
             elif self.vararg_name:
                 if self.vararg_name not in local_frame:
                     local_frame[self.vararg_name] = []
                 local_frame[self.vararg_name].append(arg)
             else:
-                raise TypeError(f"Async function '{self.node.name}' takes {num_pos} positional arguments but {len(args)} were given")
+                raise TypeError(f"Async function '{self.node.name}' takes {total_pos} positional arguments but {len(args)} were given")
         if self.vararg_name and self.vararg_name not in local_frame:
             local_frame[self.vararg_name] = tuple()
 
         for kwarg_name, kwarg_value in kwargs.items():
-            if kwarg_name in self.pos_kw_params or kwarg_name in self.kwonly_params:
+            if kwarg_name in self.posonly_params:
+                raise TypeError(f"Async function '{self.node.name}' got an unexpected keyword argument '{kwarg_name}' (positional-only)")
+            elif kwarg_name in self.pos_kw_params or kwarg_name in self.kwonly_params:
                 if kwarg_name in local_frame:
                     raise TypeError(f"Async function '{self.node.name}' got multiple values for argument '{kwarg_name}'")
                 local_frame[kwarg_name] = kwarg_value
@@ -214,7 +248,8 @@ class AsyncFunction:
             if param not in local_frame and param in self.kw_defaults:
                 local_frame[param] = self.kw_defaults[param]
 
-        missing_args = [param for param in self.pos_kw_params if param not in local_frame and param not in self.pos_defaults]
+        missing_args = [param for param in self.posonly_params if param not in local_frame]
+        missing_args += [param for param in self.pos_kw_params if param not in local_frame and param not in self.pos_defaults]
         missing_args += [param for param in self.kwonly_params if param not in local_frame and param not in self.kw_defaults]
         if missing_args:
             raise TypeError(f"Async function '{self.node.name}' missing required arguments: {', '.join(missing_args)}")
@@ -235,9 +270,10 @@ class AsyncFunction:
                 return ret.value, local_vars
             return ret.value
         finally:
-            if not _return_locals:  # Only pop if we don’t need to keep the frame
+            if not _return_locals:
                 new_env_stack.pop()
 
+
 class AsyncGeneratorFunction:
     def __init__(self, node: ast.AsyncFunctionDef, closure: List[Dict[str, Any]], interpreter: ASTInterpreter,
                  pos_kw_params: List[str], vararg_name: Optional[str], kwonly_params: List[str],
@@ -245,34 +281,45 @@ class AsyncGeneratorFunction:
         self.node: ast.AsyncFunctionDef = node
         self.closure: List[Dict[str, Any]] = closure[:]
         self.interpreter: ASTInterpreter = interpreter
+        # Fix: Support positional-only parameters
+        self.posonly_params = [arg.arg for arg in node.args.posonlyargs] if hasattr(node.args, 'posonlyargs') else []
         self.pos_kw_params = pos_kw_params
         self.vararg_name = vararg_name
         self.kwonly_params = kwonly_params
         self.kwarg_name = kwarg_name
         self.pos_defaults = pos_defaults
         self.kw_defaults = kw_defaults
-        self.generator_state = None
 
     async def __call__(self, *args: Any, **kwargs: Any) -> Any:
         new_env_stack: List[Dict[str, Any]] = self.closure[:]
         local_frame: Dict[str, Any] = {}
         local_frame[self.node.name] = self
 
-        num_pos = len(self.pos_kw_params)
+        num_posonly = len(self.posonly_params)
+        num_pos_kw = len(self.pos_kw_params)
+        total_pos = num_posonly + num_pos_kw
+        pos_args_used = 0
+
         for i, arg in enumerate(args):
-            if i < num_pos:
-                local_frame[self.pos_kw_params[i]] = arg
+            if i < num_posonly:
+                local_frame[self.posonly_params[i]] = arg
+                pos_args_used += 1
+            elif i < total_pos:
+                local_frame[self.pos_kw_params[i - num_posonly]] = arg
+                pos_args_used += 1
             elif self.vararg_name:
                 if self.vararg_name not in local_frame:
                     local_frame[self.vararg_name] = []
                 local_frame[self.vararg_name].append(arg)
             else:
-                raise TypeError(f"Async generator '{self.node.name}' takes {num_pos} positional arguments but {len(args)} were given")
+                raise TypeError(f"Async generator '{self.node.name}' takes {total_pos} positional arguments but {len(args)} were given")
         if self.vararg_name and self.vararg_name not in local_frame:
             local_frame[self.vararg_name] = tuple()
 
         for kwarg_name, kwarg_value in kwargs.items():
-            if kwarg_name in self.pos_kw_params or kwarg_name in self.kwonly_params:
+            if kwarg_name in self.posonly_params:
+                raise TypeError(f"Async generator '{self.node.name}' got an unexpected keyword argument '{kwarg_name}' (positional-only)")
+            elif kwarg_name in self.pos_kw_params or kwarg_name in self.kwonly_params:
                 if kwarg_name in local_frame:
                     raise TypeError(f"Async generator '{self.node.name}' got multiple values for argument '{kwarg_name}'")
                 local_frame[kwarg_name] = kwarg_value
@@ -290,7 +337,8 @@ class AsyncGeneratorFunction:
             if param not in local_frame and param in self.kw_defaults:
                 local_frame[param] = self.kw_defaults[param]
 
-        missing_args = [param for param in self.pos_kw_params if param not in local_frame and param not in self.pos_defaults]
+        missing_args = [param for param in self.posonly_params if param not in local_frame]
+        missing_args += [param for param in self.pos_kw_params if param not in local_frame and param not in self.pos_defaults]
         missing_args += [param for param in self.kwonly_params if param not in local_frame and param not in self.kw_defaults]
         if missing_args:
             raise TypeError(f"Async generator '{self.node.name}' missing required arguments: {', '.join(missing_args)}")
@@ -298,42 +346,92 @@ class AsyncGeneratorFunction:
         new_env_stack.append(local_frame)
         new_interp: ASTInterpreter = self.interpreter.spawn_from_env(new_env_stack)
 
-        async def generator():
-            nonlocal self
-            if self.generator_state is None:
-                self.generator_state = {"closed": False, "pending_value": None}
-                
-            for stmt in self.node.body:
-                if self.generator_state["closed"]:
+        node_body = self.node.body
+
+        class ProperAsyncGenerator:
+            def __init__(self):
+                self.running = False
+                self.exhausted = False
+                self.current_index = 0
+                self.statements = node_body
+                self.yield_from_state = {}
+                new_interp.generator_context = {
+                    'active': True,
+                    'yielded': False,
+                    'yield_value': None,
+                    'yield_from': False,
+                    'yield_from_iterable': None,
+                    'sent_value': None
+                }
+                self.interpreter = new_interp
+
+            def __aiter__(self):
+                return self
+
+            async def __anext__(self):
+                if self.exhausted:
                     raise StopAsyncIteration
-                
-                if isinstance(stmt, ast.Expr) and isinstance(stmt.value, ast.Yield):
-                    value = await new_interp.visit(stmt.value, wrap_exceptions=True)
-                    yield value
-                elif isinstance(stmt, ast.Expr) and isinstance(stmt.value, ast.YieldFrom):
-                    sub_iterable = await new_interp.visit(stmt.value, wrap_exceptions=True)
-                    async for v in sub_iterable:
-                        yield v
-                else:
-                    await new_interp.visit(stmt, wrap_exceptions=True)
+                if self.running:
+                    raise RuntimeError("AsyncGenerator already running")
+                try:
+                    self.running = True
+                    while self.current_index < len(self.statements):
+                        stmt = self.statements[self.current_index]
+                        # Fix: Properly handle raises in async generators
+                        if isinstance(stmt, ast.Raise):
+                            exc = await self.interpreter.visit(stmt, wrap_exceptions=True)
+                            raise exc
+                        await self.interpreter.visit(stmt, wrap_exceptions=True)
+                        if self.interpreter.generator_context['yielded']:
+                            self.interpreter.generator_context['yielded'] = False
+                            value = self.interpreter.generator_context['yield_value']
+                            self.current_index += 1
+                            return value
+                        self.current_index += 1
+                    self.exhausted = True
+                    raise StopAsyncIteration
+                except Exception as e:
+                    self.exhausted = True
+                    raise
+                finally:
+                    self.running = False
 
-        gen = generator()
-        gen.send = lambda value: asyncio.run_coroutine_threadsafe(self._send(gen, value), self.interpreter.loop).result()
-        gen.throw = lambda exc: asyncio.run_coroutine_threadsafe(self._throw(gen, exc), self.interpreter.loop).result()
-        gen.close = lambda: setattr(self.generator_state, "closed", True)
-        return gen
+            async def asend(self, value):
+                if self.exhausted:
+                    raise StopAsyncIteration
+                if self.running:
+                    raise RuntimeError("AsyncGenerator already running")
+                try:
+                    self.running = True
+                    self.interpreter.generator_context['sent_value'] = value
+                    return await self.__anext__()
+                finally:
+                    self.running = False
 
-    async def _send(self, gen, value):
-        try:
-            return await gen.asend(value)
-        except StopAsyncIteration:
-            raise
+            async def athrow(self, exc):
+                if self.exhausted:
+                    raise StopAsyncIteration
+                if self.running:
+                    raise RuntimeError("AsyncGenerator already running")
+                try:
+                    self.running = True
+                    raise exc
+                finally:
+                    self.running = False
+
+            async def aclose(self):
+                if self.exhausted or not self.running:
+                    return
+                try:
+                    self.running = True
+                    self.exhausted = True
+                    raise GeneratorExit("AsyncGenerator closed")
+                finally:
+                    self.running = False
+
+        # Updated: Always return the async generator object
+        return ProperAsyncGenerator()
 
-    async def _throw(self, gen, exc):
-        try:
-            return await gen.athrow(exc)
-        except StopAsyncIteration:
-            raise
 
 class LambdaFunction:
     def __init__(self, node: ast.Lambda, closure: List[Dict[str, Any]], interpreter: ASTInterpreter,
