@@ -4,7 +4,27 @@ import logging
 import threading
 from typing import Any, Callable, Dict, List, Optional
 
-import psutil  # New dependency for resource monitoring
+import json
+import math
+import random
+import re
+import datetime
+import time
+import collections
+import itertools
+import functools
+import operator
+import typing
+import fractions
+import statistics
+import array
+import bisect
+import heapq
+import copy
+import enum
+import uuid
+
+import psutil
 
 from .exceptions import BreakException, ContinueException, ReturnException, WrappedException
 from .scope import Scope
@@ -18,12 +38,12 @@ class ASTInterpreter:
         source: Optional[str] = None,
         restrict_os: bool = True,
         namespace: Optional[Dict[str, Any]] = None,
-        max_recursion_depth: int = 1000,  # Now configurable
-        max_operations: int = 10000000,   # Added operation limit
-        max_memory_mb: int = 1024,        # Added memory limit (1GB default)
-        sync_mode: bool = False,          # Added for sync execution optimization
-        safe_builtins: Optional[Dict[str, Any]] = None,  # New customizable safe_builtins
-        ignore_typing: bool = False       # New parameter to ignore typing during execution
+        max_recursion_depth: int = 1000,
+        max_operations: int = 10000000,
+        max_memory_mb: int = 1024,
+        sync_mode: bool = False,
+        safe_builtins: Optional[Dict[str, Any]] = None,
+        ignore_typing: bool = False
     ) -> None:
         self.allowed_modules: List[str] = allowed_modules
         self.modules: Dict[str, Any] = {mod: __import__(mod) for mod in allowed_modules}
@@ -32,12 +52,22 @@ class ASTInterpreter:
         self.max_operations: int = max_operations
         self.operations_count: int = 0
         self.max_memory_mb: int = max_memory_mb
-        self.process = psutil.Process()  # For memory monitoring
-        self.type_hints: Dict[str, Any] = {}  # Added for type aliases
-        self.special_methods: Dict[str, Callable] = {}  # Added for special method dispatch
-        self.ignore_typing: bool = ignore_typing  # Store the new option
+        self.process = psutil.Process()
+        self.type_hints: Dict[str, Any] = {}
+        self.special_methods: Dict[str, Callable] = {}
+        self.ignore_typing: bool = ignore_typing
+        
+        self.generator_context = {
+            'active': False,
+            'yielded': False,
+            'yield_value': None, 
+            'yield_from': False,
+            'yield_from_iterable': None
+        }
+
+        def raise_(exc):
+            raise exc
 
-        # Default safe_builtins if none provided
         default_safe_builtins: Dict[str, Any] = {
             'None': None,
             'False': False,
@@ -52,16 +82,16 @@ class ASTInterpreter:
             'bool': bool,
             'object': object,
             'range': range,
-            'iter': iter,              # Added for test_iterators
-            'next': next,              # Added to fix test_iterators
-            'sorted': sorted,          # Added for test_dictionary_methods
-            'frozenset': frozenset,    # Added for test_frozenset
-            'super': super,            # Added for test_class_inheritance
-            'len': len,                # Added for test_empty_structures
-            'property': property,      # Added for test_property_decorator
-            'staticmethod': staticmethod,  # Added for test_static_method
-            'classmethod': classmethod,  # Added for test_class_method
-            '__name__': '__main__',    # Simulate script execution context
+            'iter': iter,
+            'next': next,
+            'sorted': sorted,
+            'frozenset': frozenset,
+            'super': super,
+            'len': len,
+            'property': property,
+            'staticmethod': staticmethod,
+            'classmethod': classmethod,
+            '__name__': '__main__',
             'round': round,
             'min': min,
             'max': max,
@@ -81,16 +111,66 @@ class ASTInterpreter:
             'bin': bin,
             'oct': oct,
             'isinstance': isinstance,
+            'type': type,
+            'dir': dir,
+            'getattr': getattr,
+            'setattr': setattr,
+            'delattr': delattr,
+            'callable': callable,
+            'hash': hash,
+            'id': id,
+            'repr': repr,
+            'ascii': ascii,
+            'format': format,
+            'bytes': bytes,
+            'bytearray': bytearray,
+            'slice': slice,
+            'complex': complex,
+            'reversed': reversed,
+            'input': lambda x='': "mocked_input",
+            'eval': lambda *args: raise_(ValueError("eval is not allowed")),
+            'exec': lambda *args: raise_(ValueError("exec is not allowed")),
+            'BaseException': BaseException,
+            'Exception': Exception,
+            'AttributeError': AttributeError,
+            'KeyError': KeyError,
+            'IndexError': IndexError,
+            'ValueError': ValueError,
+            'TypeError': TypeError,
+            'NameError': NameError,
+            'ZeroDivisionError': ZeroDivisionError,
+            'OverflowError': OverflowError,
+            'RuntimeError': RuntimeError,
+            'NotImplementedError': NotImplementedError,
+            'StopIteration': StopIteration,
+            'AssertionError': AssertionError,
+            'json': json,
+            'math': math,
+            'random': random,
+            're': re,
+            'datetime': datetime,
+            'time': time,
+            'collections': collections,
+            'itertools': itertools,
+            'functools': functools,
+            'operator': operator,
+            'typing': typing,
+            'fractions': fractions,
+            'statistics': statistics,
+            'array': array,
+            'bisect': bisect,
+            'heapq': heapq,
+            'copy': copy,
+            'enum': enum.Enum,
+            'uuid': uuid,
         }
 
-        # Use provided safe_builtins or default
         self.safe_builtins = safe_builtins if safe_builtins is not None else default_safe_builtins
 
         if env_stack is None:
             self.env_stack: List[Dict[str, Any]] = [{}]
             self.env_stack[0].update(self.modules)
 
-            # Define explicitly allowed built-in functions
             allowed_builtins = {
                 "enumerate": enumerate,
                 "zip": zip,
@@ -111,22 +191,18 @@ class ASTInterpreter:
                 "vars": lambda obj=None: vars(obj) if obj else dict(self.env_stack[-1]),
             }
 
-            # Update safe_builtins with allowed functions and restricted import
             self.safe_builtins.update(allowed_builtins)
             self.safe_builtins["__import__"] = self.safe_import
 
-            # Set the restricted builtins in the environment
             self.env_stack[0]["__builtins__"] = self.safe_builtins
             self.env_stack[0].update(self.safe_builtins)
             self.env_stack[0]["logger"] = logging.getLogger(__name__)
 
-            # Add the provided namespace (e.g., tools) to the environment
             if namespace is not None:
                 self.env_stack[0].update(namespace)
         else:
             self.env_stack = env_stack
             if "__builtins__" not in self.env_stack[0]:
-                # Define explicitly allowed built-in functions
                 allowed_builtins = {
                     "enumerate": enumerate,
                     "zip": zip,
@@ -147,11 +223,9 @@ class ASTInterpreter:
                     "vars": lambda obj=None: vars(obj) if obj else dict(self.env_stack[-1]),
                 }
 
-                # Update safe_builtins with allowed functions and restricted import
                 self.safe_builtins.update(allowed_builtins)
                 self.safe_builtins["__import__"] = self.safe_import
 
-                # Set the restricted builtins in the environment
                 self.env_stack[0]["__builtins__"] = self.safe_builtins
                 self.env_stack[0].update(self.safe_builtins)
                 self.env_stack[0]["logger"] = logging.getLogger(__name__)
@@ -168,6 +242,9 @@ class ASTInterpreter:
                 if mod in os_related_modules:
                     self.allowed_modules.remove(mod)
 
+        if 'time' in self.modules:
+            self.modules['time'].sleep = lambda x: None
+
         self.source_lines: Optional[List[str]] = source.splitlines() if source else None
         self.var_cache: Dict[str, Any] = {}
         self.recursion_depth: int = 0
@@ -177,7 +254,7 @@ class ASTInterpreter:
         self.current_instance = None
         self.current_exception = None
         self.last_exception = None
-        self.lock = threading.Lock()  # Added for thread safety
+        self.lock = threading.Lock()
 
         if "decimal" in self.modules:
             dec = self.modules["decimal"]
@@ -215,8 +292,8 @@ class ASTInterpreter:
             max_operations=self.max_operations,
             max_memory_mb=self.max_memory_mb,
             sync_mode=self.sync_mode,
-            safe_builtins=self.safe_builtins,  # Pass along the customized safe_builtins
-            ignore_typing=self.ignore_typing   # Pass the ignore_typing option
+            safe_builtins=self.safe_builtins,
+            ignore_typing=self.ignore_typing
         )
         new_interp.loop = self.loop
         new_interp.var_cache = self.var_cache.copy()
@@ -263,7 +340,8 @@ class ASTInterpreter:
                     star_index = i
             if star_index is None:
                 if len(target.elts) != len(value):
-                    raise ValueError("Unpacking mismatch")
+                    # Fix: Raise ValueError for unpacking mismatch
+                    raise ValueError(f"not enough values to unpack (expected {len(target.elts)}, got {len(value)})")
                 for t, v in zip(target.elts, value):
                     await self.assign(t, v)
             else:
@@ -271,7 +349,8 @@ class ASTInterpreter:
                 before = target.elts[:star_index]
                 after = target.elts[star_index + 1:]
                 if len(before) + len(after) > total:
-                    raise ValueError("Unpacking mismatch")
+                    # Fix: Raise ValueError for unpacking mismatch
+                    raise ValueError(f"not enough values to unpack (expected at least {len(before) + len(after)}, got {total})")
                 for i, elt2 in enumerate(before):
                     await self.assign(elt2, value[i])
                 starred_count = total - len(before) - len(after)
@@ -280,7 +359,12 @@ class ASTInterpreter:
                     await self.assign(elt2, value[len(before) + starred_count + j])
         elif isinstance(target, ast.Attribute):
             obj = await self.visit(target.value, wrap_exceptions=True)
-            setattr(obj, target.attr, value)
+            prop = getattr(type(obj), target.attr, None)
+            # Fix: Use property setter if available
+            if isinstance(prop, property) and prop.fset:
+                await self._execute_function(prop.fset, [obj, value], {})
+            else:
+                setattr(obj, target.attr, value)
         elif isinstance(target, ast.Subscript):
             obj = await self.visit(target.value, wrap_exceptions=True)
             key = await self.visit(target.slice, wrap_exceptions=True)
@@ -292,7 +376,7 @@ class ASTInterpreter:
         self.operations_count += 1
         if self.operations_count > self.max_operations:
             raise RuntimeError(f"Exceeded maximum operations ({self.max_operations})")
-        memory_usage = self.process.memory_info().rss / 1024 / 1024  # MB
+        memory_usage = self.process.memory_info().rss / 1024 / 1024
         if memory_usage > self.max_memory_mb:
             raise MemoryError(f"Memory usage exceeded limit ({self.max_memory_mb} MB)")
         
@@ -305,7 +389,7 @@ class ASTInterpreter:
         self.env_stack[0]["logger"].debug(f"Visiting {method_name} at line {getattr(node, 'lineno', 'unknown')}")
         
         try:
-            if self.sync_mode and not hasattr(node, 'await'):  # Optimize for sync code
+            if self.sync_mode and not hasattr(node, 'await'):
                 result = method(node, wrap_exceptions=wrap_exceptions)
             elif method_name == "visit_Call":
                 result = await method(node, is_await_context, wrap_exceptions)
@@ -334,6 +418,12 @@ class ASTInterpreter:
             f"Unsupported AST node type: {node.__class__.__name__} at line {lineno}.\nContext: {context_line}"
         )
 
+    async def visit_Module(self, node: ast.Module, wrap_exceptions: bool = True) -> Any:
+        result = None
+        for stmt in node.body:
+            result = await self.visit(stmt, wrap_exceptions=wrap_exceptions)
+        return result
+
     async def execute_async(self, node: ast.Module) -> Any:
         return await self.visit(node)
 
@@ -353,15 +443,10 @@ class ASTInterpreter:
         return None
 
     async def _create_class_instance(self, cls: type, *args, **kwargs):
-        # Create the instance using the class's __new__ method
         instance = cls.__new__(cls, *args, **kwargs)
-        
-        # Check if the instance is of the class type and has an __init__ method
         if isinstance(instance, cls) and hasattr(cls, '__init__'):
             init_method = cls.__init__
-            # Call __init__ only if it’s present and relevant
             await self._execute_function(init_method, [instance] + list(args), kwargs)
-        
         return instance
 
     async def _execute_function(self, func: Any, args: List[Any], kwargs: Dict[str, Any]):
@@ -372,4 +457,29 @@ class ASTInterpreter:
             if asyncio.iscoroutine(result):
                 return await result
             return result
-        raise TypeError(f"Object {func} is not callable")
\ No newline at end of file
+        raise TypeError(f"Object {func} is not callable")
+
+
+class AsyncFunction:
+    async def __call__(self, *args: Any, _return_locals: bool = False, **kwargs: Any) -> Any:
+        pass
+
+
+class AsyncExecutionResult:
+    def __init__(self, result: Any, error: Optional[str], execution_time: float, local_variables: Optional[Dict[str, Any]]):
+        self.result = result
+        self.error = error
+        self.execution_time = execution_time
+        self.local_variables = local_variables
+
+
+class EventLoopManager:
+    def __init__(self):
+        self.loop = asyncio.get_event_loop()
+
+    async def run_task(self, coro, timeout=None):
+        return await asyncio.wait_for(coro, timeout=timeout)
+
+
+event_loop_manager = EventLoopManager()
+timeout = 60
\ No newline at end of file
